@@ -35,7 +35,7 @@
 #include "file.h"
 
 #ifndef lint
-FILE_RCSID("@(#)$File: cdf.c,v 1.116 2019/08/26 14:31:39 christos Exp $")
+FILE_RCSID("@(#)$File: cdf.c,v 1.121 2021/10/20 13:56:15 christos Exp $")
 #endif
 
 #include <assert.h>
@@ -48,6 +48,12 @@ FILE_RCSID("@(#)$File: cdf.c,v 1.116 2019/08/26 14:31:39 christos Exp $")
 #include <time.h>
 #include <ctype.h>
 #include <limits.h>
+#ifdef HAVE_BYTESWAP_H
+#include <byteswap.h>
+#endif
+#ifdef HAVE_SYS_BSWAP_H
+#include <sys/bswap.h>
+#endif
 
 #ifndef EFTYPE
 #define EFTYPE EINVAL
@@ -97,6 +103,8 @@ cdf_malloc(const char *file __attribute__((__unused__)),
 {
 	DPRINTF(("%s,%" SIZE_T_FORMAT "u: %s %" SIZE_T_FORMAT "u\n",
 	    file, line, __func__, n));
+	if (n == 0)
+	    n++;
 	return malloc(n);
 }
 
@@ -117,9 +125,20 @@ cdf_calloc(const char *file __attribute__((__unused__)),
 {
 	DPRINTF(("%s,%" SIZE_T_FORMAT "u: %s %" SIZE_T_FORMAT "u %"
 	    SIZE_T_FORMAT "u\n", file, line, __func__, n, u));
+	if (n == 0)
+	    n++;
 	return calloc(n, u);
 }
 
+#if defined(HAVE_BYTESWAP_H)
+# define _cdf_tole2(x)	bswap_16(x)
+# define _cdf_tole4(x)	bswap_32(x)
+# define _cdf_tole8(x)	bswap_64(x)
+#elif defined(HAVE_SYS_BSWAP_H)
+# define _cdf_tole2(x)	bswap16(x)
+# define _cdf_tole4(x)	bswap32(x)
+# define _cdf_tole8(x)	bswap64(x)
+#else
 /*
  * swap a short
  */
@@ -169,6 +188,7 @@ _cdf_tole8(uint64_t sv)
 	d[7] = s[0];
 	return rv;
 }
+#endif
 
 /*
  * grab a uint32_t from a possibly unaligned address, and return it in
@@ -888,29 +908,29 @@ cdf_get_property_info_pos(const cdf_stream_t *sst, const cdf_header_t *h,
 {
 	size_t tail = (i << 1) + 1;
 	size_t ofs;
-	const uint8_t *q;
 
 	if (p >= e) {
 		DPRINTF(("Past end %p < %p\n", e, p));
 		return NULL;
 	}
+
 	if (cdf_check_stream_offset(sst, h, p, (tail + 1) * sizeof(uint32_t),
 	    __LINE__) == -1)
 		return NULL;
+
 	ofs = CDF_GETUINT32(p, tail);
-	q = CAST(const uint8_t *, cdf_offset(CAST(const void *, p),
-	    ofs - 2 * sizeof(uint32_t)));
-
-	if (q < p) {
-		DPRINTF(("Wrapped around %p < %p\n", q, p));
+	if (ofs < 2 * sizeof(uint32_t)) {
+		DPRINTF(("Offset too small %zu\n", ofs));
 		return NULL;
 	}
 
-	if (q >= e) {
-		DPRINTF(("Ran off the end %p >= %p\n", q, e));
+	ofs -= 2 * sizeof(uint32_t);
+	if (ofs > CAST(size_t, e - p)) {
+		DPRINTF(("Offset too big %zu %td\n", ofs, e - p));
 		return NULL;
 	}
-	return q;
+
+	return CAST(const uint8_t *, cdf_offset(CAST(const void *, p), ofs));
 }
 
 static cdf_property_info_t *
@@ -1090,8 +1110,9 @@ cdf_read_property_info(const cdf_stream_t *sst, const cdf_header_t *h,
 
 				DPRINTF(("o=%" SIZE_T_FORMAT "u l=%d(%"
 				    SIZE_T_FORMAT "u), t=%" SIZE_T_FORMAT
-				    "u s=%s\n", o4, l, CDF_ROUND(l, sizeof(l)),
-				    left, inp[i].pi_str.s_buf));
+				    "u s=%.*s\n", o4, l,
+				    CDF_ROUND(l, sizeof(l)),
+				    left, (int)l, inp[i].pi_str.s_buf));
 
 				if (l & 1)
 					l++;
